==> git-232.9559.62-sources.jar: git4idea/repo/GitRecentCheckoutBranches.kt <==
// Copyright 2000-2023 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
@file:JvmName("GitRecentCheckoutBranches")
package git4idea.repo

import com.intellij.openapi.util.registry.Registry
import com.intellij.util.concurrency.annotations.RequiresBackgroundThread
import git4idea.GitLocalBranch
import git4idea.commands.Git
import git4idea.commands.GitCommand
import git4idea.commands.GitLineHandler
import git4idea.config.GitVcsSettings

/**
 * Collect the first "git.recent.checkout.branches.reflog.entries.max.count" checkout branch entries from reflog.
 * An initial branch after git clone isn't stored as "checkout" entry in reflog,
 * [GitVcsSettings.getRecentBranchesByRepository] will be used instead.
 */
@RequiresBackgroundThread
fun GitRepository.collectRecentCheckoutBranches(haveLocalBranch: (GitLocalBranch) -> Boolean): List<GitLocalBranch> {
  val repo = this
  val recentBranchFromSettings = GitVcsSettings.getInstance(project).recentBranchesByRepository[root.path]?.let(::GitLocalBranch)
  val reflogEntriesCount = Registry.intValue("git.recent.checkout.branches.reflog.entries.max.count")
  if (reflogEntriesCount <= 0) return emptyList()

  val handler = GitLineHandler(repo.project, repo.root, GitCommand.REF_LOG).apply {
    setSilent(true)
    addParameters("--max-count", reflogEntriesCount.toString(), "--grep-reflog", "checkout:")
    endOptions()
  }
  val result = Git.getInstance().runCommand(handler)
  if (!result.success()) return emptyList()

  val branches = linkedSetOf<GitLocalBranch>()
  var recentBranchFromSettingsAdded = false
  val toClause = " to "
  for (line in result.output) {
    val toIndex = line.lastIndexOf(toClause)
    if (toIndex <= 0) continue

    val branchName = line.substring(toIndex + toClause.length, line.length)

    val localBranch = GitLocalBranch(branchName)
    if (haveLocalBranch(localBranch)) {
      if (recentBranchFromSettings?.name == branchName) {
        recentBranchFromSettingsAdded = true
      }
      branches.add(localBranch)
    }
  }

  if (recentBranchFromSettings != null && !recentBranchFromSettingsAdded && haveLocalBranch(recentBranchFromSettings)) {
    branches.add(recentBranchFromSettings)
  }

  return branches.toList()
}

==> git-233.14808.21-sources.jar: git4idea/repo/GitRecentCheckoutBranches.kt <==
// Copyright 2000-2023 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
@file:JvmName("GitRecentCheckoutBranches")
package git4idea.repo

import com.intellij.openapi.util.registry.Registry
import com.intellij.util.concurrency.annotations.RequiresBackgroundThread
import git4idea.GitLocalBranch
import git4idea.commands.Git
import git4idea.commands.GitCommand
import git4idea.commands.GitLineHandler
import git4idea.config.GitVcsSettings

/**
 * Collect the first "git.recent.checkout.branches.reflog.entries.count" checkout branch entries from reflog.
 * An initial branch after git clone isn't stored as "checkout" entry in reflog,
 * [GitVcsSettings.getRecentBranchesByRepository] will be used instead.
 */
@RequiresBackgroundThread
fun GitRepository.collectRecentCheckoutBranches(haveLocalBranch: (GitLocalBranch) -> Boolean): List<GitLocalBranch> {
  val repo = this
  val recentBranchFromSettings = GitVcsSettings.getInstance(project).recentBranchesByRepository[root.path]?.let(::GitLocalBranch)
  val reflogEntriesCount = Registry.intValue("git.recent.checkout.branches.reflog.entries.count")
  if (reflogEntriesCount <= 0) return emptyList()

  val handler = GitLineHandler(repo.project, repo.root, GitCommand.REF_LOG).apply {
    setSilent(true)
    addParameters("--max-count", reflogEntriesCount.toString(), "--grep-reflog", "checkout:")
    endOptions()
  }
  val result = Git.getInstance().runCommand(handler)
  if (!result.success()) return emptyList()

  val branches = linkedSetOf<GitLocalBranch>()
  var recentBranchFromSettingsAdded = false
  val toClause = " to "
  for (line in result.output) {
    val toIndex = line.lastIndexOf(toClause)
    if (toIndex <= 0) continue

    val branchName = line.substring(toIndex + toClause.length, line.length)

    val localBranch = GitLocalBranch(branchName)
    if (haveLocalBranch(localBranch)) {
      if (recentBranchFromSettings?.name == branchName) {
        recentBranchFromSettingsAdded = true
      }
      branches.add(localBranch)
    }
  }

  if (recentBranchFromSettings != null && !recentBranchFromSettingsAdded && haveLocalBranch(recentBranchFromSettings)) {
    branches.add(recentBranchFromSettings)
  }

  return branches.toList()
}
